from mqtt_base_device import MqttBaseDevice


class MqttBaseListener(MqttBaseDevice):

    def __init__(self, server="test.mosquitto.org",
                 topic="/service/%s" % 'c8fc7f8f-53d4-4457-9da1-478e99b1c87f'):
        super(MqttBaseListener, self).__init__(server)
        self.client = None
        self.topic = topic

    # The callback for when the client receives a CONNACK response from the server.
    def on_connect(self, client, user_data, flags, rc):
        super(MqttBaseListener, self).on_connect(client, user_data, flags, rc)

        # Subscribing in on_connect() means that if we lose the connection and
        # reconnect then subscriptions will be renewed.
        self.client = client
        self.client.subscribe(self.topic)
        print "subscribed to:", self.get_topic_name()

    def get_topic_name(self):
        return self.topic

    # The callback for when a PUBLISH message is received from the server.
    def on_message(self, client, user_data, msg):
        """

        :param client:
        :param user_data:
        :param msg: msg.topic is the topic of the message
        msg.payload is the payload of the message
        :return: None
        """
        pass



class MqttTopicSchema(object):
    """
    Why to use 2 different topics for request and event?
    If use the same topic, device will also receive the event sent by itself.

    """

    topic_root_name = "smart-dev"
    request_topic_name = "request"
    event_topic_name = "event"
    topic_for_device_path_pattern = "%s/%s"
    topic_root_pattern_for_dev_under_gw = "/%s/%s/%s"
    topic_root_pattern_for_gw = "/%s/%s"

    def __init__(self, parent_device_id=None):
        super(MqttTopicSchema, self).__init__()

        if parent_device_id is None:
            self.request_topic_root = self.topic_root_pattern_for_gw % (self.topic_root_name, self.request_topic_name)
            self.event_topic_root = self.topic_root_pattern_for_gw % (self.topic_root_name, self.event_topic_name)
        else:
            self.request_topic_root = self.topic_root_pattern_for_dev_under_gw % (
                self.topic_root_name,
                self.request_topic_name, parent_device_id)
            self.event_topic_root = self.topic_root_pattern_for_dev_under_gw % (
                self.topic_root_name, self.event_topic_name, parent_device_id)
            self.parent_device_id = parent_device_id

    def get_event_channel_name(self, device_id):
        return self.topic_for_device_path_pattern % (self.event_topic_root, device_id)

    def get_request_channel_name(self, device_id):
        return self.topic_for_device_path_pattern % (self.request_topic_root, device_id)

    def get_gateway_request_channel_name(self):
        return self.request_topic_root

    # noinspection PyMethodMayBeStatic
    def get_device_id_from_topic(self, topic):
        topic_paths = topic.split("/")
        return topic_paths[-1]

    def get_last_will_topic(self, device_id):
        return self.get_event_channel_name(device_id)

    def get_gw_event_channel_name(self):
        return "%s/" % self.event_topic_root

    def get_gw_event_channel_wildcard(self):
        return "%s/#" % self.event_topic_root

    def is_gateway_message(self, topic):
        msg_topic = self.get_device_id_from_topic(topic.strip("/"))
        return msg_topic == self.parent_device_id

    def get_custom_action_topic_for_gw(self, action_name):
        return "/{topic_root_name}/{action_name}/{gw_id}".format(
            topic_root_name=self.topic_root_name,
            action_name=action_name,
            gw_id=self.parent_device_id)



from mqtt_base_listener import MqttBaseListener


class MqttMultiTopicListener(MqttBaseListener):

    def __init__(self, server="test.mosquitto.org", listening_channel_list=None):
        super(MqttMultiTopicListener, self).__init__(server)
        if listening_channel_list is None:
            listening_channel_list = []
        self.listening_channel_list = listening_channel_list

    # The callback for when the client receives a CONNACK response from the server.
    def on_connect(self, client, user_data, flags, rc):
        print("Connected with result code " + str(rc))
        # Subscribing in on_connect() means that if we lose the connection and
        # reconnect then subscriptions will be renewed.
        for channel_name in self.get_listening_channel_list():
            print "subscribed to:", channel_name
            client.subscribe(channel_name)

    def get_listening_channel_list(self):
        return self.listening_channel_list



import paho.mqtt.client as mqtt


class MqttBaseDevice(object):

    def __init__(self, server="test.mosquitto.org"):
        super(MqttBaseDevice, self).__init__()
        self.client = None
        self.topic = "/service/%s" % 'c8fc7f8f-53d4-4457-9da1-478e99b1c87f'
        self.server = server
        self.last_will_topic = None
        self.last_will = None

    def loop_start(self):
        self.create_client()
        self.client.loop_start()

    def loop_forever(self):
        self.create_client()
        self.client.loop_forever()

    def create_client(self):
        self.client = mqtt.Client()
        if self.last_will_topic is not None:
            self.client.will_set(self.last_will_topic, self.last_will)
        self.client.on_connect = self.on_connect
        self.client.on_message = self.on_message
        self.client.connect(self.server, 1883, 60)
        # self.client.connect("iot.eclipse.org", 1883, 60)
        # self.client.connect("192.168.8.1", 1883, 60)

    def on_message(self, client, user_data, msg):
        pass

    # def _on_connect(self, client, userdata, flags, rc):
    #     self.on_connect(client, userdata, flags, rc)
    #
    # def _on_message(self, client, userdata, msg):
    #     self.on_message(client, userdata, msg)

    # The callback for when the client receives a CONNACK response from the server.
    def on_connect(self, client, user_data, flags, rc):
        print("Connected with result code " + str(rc))
